/******************************************************************************
 FIFO Example Program

 This program creates two tasks and a FIFO. Messages are sent  from one task 
 to another using the FIFO. A timeout value for the pend function is used to
 illustrate the timeout feature of the FIFO.

 *****************************************************************************/
#include "predef.h"
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <ucos.h>
#include <serial.h>
#include <dhcpclient.h>
#include <netinterface.h>


#define NUM_POSTS (5)

//----- function prototypes -----
extern "C"
{
   void UserMain( void *pd );
}


//----- structs -----
typedef struct // Pointers to this structure will be passed in the FIFO
{
      void * pUsedByFifo;  // Don't modify this value, and keep it first
      int    x;            // Any other members come after the pointer
      BOOL   free;         // free = TRUE if structure is free to be used.
} MyStructure;


//----- global vars -----
DWORD FifoPostTaskStack[USER_TASK_STK_SIZE];
OS_FIFO MyFIFO;
MyStructure StructArray[5];


/*-------------------------------------------------------------------
 Init FIFO members
 ------------------------------------------------------------------*/
void InitFifoMembers()
{
   for ( int i = 0; i < NUM_POSTS; i++ )
   {
      StructArray[i].x = 0;
      StructArray[i].free = TRUE;
   }
}


/*-------------------------------------------------------------------
 Find free FIFO structure.
 Rather than use dynamic memory allocation, this example uses a 
 declared array of structures with a structure element that indicates
 if a particular structure is being used. This function will return
 the index of the first free structure array member, or -1 if all
 members are being used.
 ------------------------------------------------------------------*/
int FindFreeFifoStruct( )
{
   int index = -1;
   int i = 0;

   USER_ENTER_CRITICAL();
   do {
      if ( StructArray[i].free )
         index = i;
      else
         i++;
   } while ( ( i < NUM_POSTS ) && ( index == -1 ) );
   StructArray[i].free = FALSE;   
   USER_EXIT_CRITICAL();
   
   return index;   
}


/*-------------------------------------------------------------------
 FIFO Post Task
 ------------------------------------------------------------------*/
void FifoPostTask( void *pdata )
{
   while ( 1 )
   {
      iprintf("\r\n");
      for ( int count = 0; count < NUM_POSTS; count++ )
      {
         int i;
         while ( (i = FindFreeFifoStruct() ) < 0 )
         {
            iprintf("Waiting for free FIFO structure\r\n");
            OSTimeDly( TICKS_PER_SECOND / 2 );
         }
         
         StructArray[i].x = count;
         OSFifoPost( &MyFIFO, ( OS_FIFO_EL * ) &StructArray[i] ); // Put a message in the Fifo.
         iprintf( "     Posted FIFO value %d\r\n", i );
      }
      iprintf("\r\n");

      OSTimeDly( TICKS_PER_SECOND * 9 );
   }
}


/*-------------------------------------------------------------------
 UserMain
 ------------------------------------------------------------------*/
void UserMain( void *pd )
{
   DWORD FifoCnt = 0;

   InitializeStack();
   
   int EthernetInterface = GetFirstInterface();  // Get first interface identifier
   InterfaceBlock *ib = GetInterFaceBlock( EthernetInterface );  // Get interface data
  
   if (ib->netIP == 0)  // Check IP address for 0.0.0.0, and use DHCP if necessary
   {
      iprintf("\r\nNo static IP address set, attempting DHCP\r\n");
      if ( GetDHCPAddress() != DHCP_OK )
         iprintf("*** DHCP Failed \r\n");
      else
         iprintf( "DHCP IP Address: ");
   }
   else
   {
      iprintf("Static IP Address: ");
   }
   ShowIP(ib->netIP);iprintf("/r/n");

   OSChangePrio( MAIN_PRIO );
   EnableAutoUpdate();

   OSFifoInit( &MyFIFO );
   InitFifoMembers();

   if ( OSTaskCreate( FifoPostTask,
                      NULL,
                      ( void * ) &FifoPostTaskStack[USER_TASK_STK_SIZE],
                      ( void * ) FifoPostTaskStack,
                      MAIN_PRIO - 1 ) != OS_NO_ERR )
   {
      iprintf( "*** Error creating task\r\n" );
   }

   while ( 1 )
   {
      iprintf( ">>> Calling OSFifoPend()...\r\n" );
      MyStructure *pData = ( MyStructure * ) OSFifoPend( &MyFIFO, TICKS_PER_SECOND * 5 );
      if ( pData == NULL )
      {
         // Because of the OSTimeDly() in the post task, we will timeout
         // once per sequence on purpose.
         iprintf( "    Timeout in OSFifoPend()\r\n" );
      }
      else
      {
         iprintf( "    FIFO Read #%d, value: %d\r\n", FifoCnt++, pData->x );
            pData->free = TRUE;   
      }
   }
}








